<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      Roland Dalmulder <contact@csvimproved.com>
 * @copyright   Copyright (C) 2006 - 2016 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        http://www.csvimproved.com
 */

defined('_JEXEC') or die;

require_once JPATH_ADMINISTRATOR . '/components/com_csvi/models/exports.php';

/**
 * Export HikaShop order.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class Com_HikashopModelExportOrder extends CsviModelExports
{
	/**
	 * Export the data.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 *
	 * @throws  CsviException
	 * @throws  RuntimeException
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Build something fancy to only get the fieldnames the user wants
			$userfields = array();
			$exportfields = $this->fields->getFields();

			// Include category helper
			require_once JPATH_ADMINISTRATOR . '/components/com_csvi/addon/com_hikashop/helper/com_hikashop_category_product.php';
			$categoryHelper = new Com_HikashopHelperCom_Hikashop_Category_Product($this->db, $this->template, $this->log);

			// Group by fields
			$groupbyfields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupby = array();

			if (isset($groupbyfields->name))
			{
				$groupbyfields = array_flip($groupbyfields->name);
			}
			else
			{
				$groupbyfields = array();
			}

			// Sort selected fields
			$sortfields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortby = array();

			if (isset($sortfields->name))
			{
				$sortbyfields = array_flip($sortfields->name);
			}
			else
			{
				$sortbyfields = array();
			}

			foreach ($exportfields as $field)
			{
				switch ($field->field_name)
				{
					case 'order_id':
						$userfields[] = $this->db->quoteName('o.order_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('o.order_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('o.order_id');
						}
						break;
					case 'product_id':
						$userfields[] = $this->db->quoteName('p.product_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('p.product_id');

						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('p.product_id');
						}
						break;
					case 'category_path':
						$userfields[] = $this->db->quoteName('pc.category_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('pc.category_id');

						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('pc.category_id');
						}
						break;
					case 'custom':
						break;
					default:
						// Check if it is a billing or shipping field
						if (0 === strpos($field->field_name, 'billing'))
						{
							$userfields[] = $this->db->quoteName('ab.' . str_ireplace('billing_', '', $field->field_name), $field->field_name);

							if (array_key_exists($field->field_name, $groupbyfields))
							{
								$groupby[] = $this->db->quoteName('ab.' . $field->field_name);
							}

							if (array_key_exists($field->field_name, $sortbyfields))
							{
								$sortby[] = $this->db->quoteName('ab.' . $field->field_name);
							}
						}
						elseif (0 === strpos($field->field_name, 'shipping'))
						{
							$userfields[] = $this->db->quoteName('as.' . str_ireplace('shipping_', '', $field->field_name), $field->field_name);

							if (array_key_exists($field->field_name, $groupbyfields))
							{
								$groupby[] = $this->db->quoteName('as.' . $field->field_name);
							}

							if (array_key_exists($field->field_name, $sortbyfields))
							{
								$sortby[] = $this->db->quoteName('as.' . $field->field_name);
							}
						}
						else
						{
							$userfields[] = $this->db->quoteName($field->field_name);

							if (array_key_exists($field->field_name, $groupbyfields))
							{
								$groupby[] = $this->db->quoteName($field->field_name);
							}

							if (array_key_exists($field->field_name, $sortbyfields))
							{
								$sortby[] = $this->db->quoteName($field->field_name);
							}
						}
						break;
				}
			}

			// Build the query
			$userfields = array_unique($userfields);
			$query = $this->db->getQuery(true)
				->select(implode(",\n", $userfields))
				->from($this->db->quoteName('#__hikashop_order', 'o'))
				->leftJoin(
						$this->db->quoteName('#__hikashop_order_product', 'p')
						. ' ON ' . $this->db->quoteName('p.order_id') . ' = ' . $this->db->quoteName('o.order_id')
				)
				->leftJoin(
					$this->db->quoteName('#__hikashop_user', 'u')
					. ' ON ' . $this->db->quoteName('u.user_id') . ' = ' . $this->db->quoteName('o.order_user_id')
				)
				->leftJoin(
					$this->db->quoteName('#__hikashop_address', 'ab')
					. ' ON ' . $this->db->quoteName('ab.address_id') . ' = ' . $this->db->quoteName('o.order_billing_address_id')
				)
				->leftJoin(
					$this->db->quoteName('#__hikashop_address', 'as')
					. ' ON ' . $this->db->quoteName('as.address_id') . ' = ' . $this->db->quoteName('o.order_shipping_address_id')
				)
				->leftJoin(
					$this->db->quoteName('#__hikashop_product', 'prod')
					. ' ON ' . $this->db->quoteName('prod.product_code') . ' = ' . $this->db->quoteName('p.order_product_code')
				)
				->leftJoin(
					$this->db->quoteName('#__users', 'user')
					. ' ON ' . $this->db->quoteName('user.id') . ' = ' . $this->db->quoteName('u.user_cms_id')
				)
				->leftJoin(
					$this->db->quoteName('#__hikashop_product_category', 'pc')
					. ' ON ' . $this->db->quoteName('pc.product_id') . ' = ' . $this->db->quoteName('prod.product_id')
				)
				->leftJoin(
				$this->db->quoteName('#__hikashop_category', 'cat')
				. ' ON ' . $this->db->quoteName('cat.category_id') . ' = ' . $this->db->quoteName('pc.category_id')
				);

			// Filter by order product category
			$category = $this->template->get('ordercategory', false);

			if ($category && $category[0] !== 'none')
			{
				array_shift($category);
				$query->where($this->db->quoteName('pc.category_id') . ' IN (' . implode(',', $category) . ')');
			}

			// Filter by brands
			$manufacturer = $this->template->get('ordermanufacturer', false);

			if ($manufacturer && $manufacturer[0] !== 'none')
			{
				$query->where($this->db->quoteName('prod.product_manufacturer_id') . ' IN (' . implode(',', $manufacturer) . ')');
			}

			// Filter by order number start
			$ordernostart = $this->template->get('ordernostart', 0, 'int');

			if ($ordernostart > 0)
			{
				$query->where($this->db->quoteName('o.order_id') . ' >= ' . (int) $ordernostart);
			}

			// Filter by order number end
			$ordernoend = $this->template->get('ordernoend', 0, 'int');

			if ($ordernoend > 0)
			{
				$query->where($this->db->quoteName('o.order_id') . ' <= ' . (int) $ordernoend);
			}

			// Filter by list of order numbers
			$orderlist = $this->template->get('orderlist');

			if ($orderlist)
			{
				$query->where($this->db->quoteName('o.order_id') . ' IN (' . $orderlist . ')');
			}

			// Check for a pre-defined date
			$daterange = $this->template->get('orderdaterange', '');

			if ($daterange != '')
			{
				$jdate = JFactory::getDate();
				$dateformat = '%Y-%m-%d';

				switch ($daterange)
				{
					case 'lastrun':
						if (substr($this->template->getLastrun(), 0, 4) != '0000')
						{
							$lastrun = strtotime($this->template->getLastrun());
							$query->where($this->db->quoteName('o.order_created') . ' > ' . $this->db->quote($lastrun));
						}
						break;
					case 'yesterday':
						$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') = DATE_SUB(CURDATE(), INTERVAL 1 DAY)');
						break;
					case 'thisweek':
						// Get the current day of the week
						$dayofweek = $jdate->__get('dayofweek');
						$offset = $dayofweek - 1;
						$query->where(
							'FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\')
							>= DATE_SUB(CURDATE(), INTERVAL ' . $offset . ' DAY)'
						);
						$query->where(
							'FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') <= CURDATE()'
						);
						break;
					case 'lastweek':
						// Get the current day of the week
						$dayofweek = $jdate->__get('dayofweek');
						$offset = $dayofweek + 6;
						$query->where(
							'FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\')
							>= DATE_SUB(CURDATE(), INTERVAL ' . $offset . ' DAY)'
						);
						$query->where(
							'FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\')
							<= DATE_SUB(CURDATE(), INTERVAL ' . $dayofweek . ' DAY)'
						);
						break;
					case 'thismonth':
						// Get the current day of the week
						$dayofmonth = $jdate->__get('day');
						$query->where(
							'FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\')
							>= DATE_SUB(CURDATE(), INTERVAL ' . $dayofmonth . ' DAY)'
						);
						$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') <= CURDATE()');
						break;
					case 'lastmonth':
						// Get the current day of the week
						$dayofmonth = $jdate->__get('day');
						$month = date('n');
						$year = date('y');

						if ($month > 1)
						{
							$month--;
						}
						else
						{
							$month = 12;
							$year--;
						}

						$daysinmonth = date('t', mktime(0, 0, 0, $month, 25, $year));
						$offset = ($daysinmonth + $dayofmonth) - 1;

						$query->where(
							'FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\')
							>= DATE_SUB(CURDATE(), INTERVAL ' . $offset . ' DAY)'
						);
						$query->where(
							'FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\')
							<= DATE_SUB(CURDATE(), INTERVAL ' . $dayofmonth . ' DAY)'
						);
						break;
					case 'thisquarter':
						// Find out which quarter we are in
						$month = $jdate->__get('month');
						$year = date('Y');
						$quarter = ceil($month / 3);

						switch ($quarter)
						{
							case '1':
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') >= ' . $this->db->quote($year . '-01-01'));
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') < ' . $this->db->quote($year . '-04-01'));
								break;
							case '2':
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') >= ' . $this->db->quote($year . '-04-01'));
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') < ' . $this->db->quote($year . '-07-01'));
								break;
							case '3':
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') >= ' . $this->db->quote($year . '-07-01'));
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') < ' . $this->db->quote($year . '-10-01'));
								break;
							case '4':
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') >= ' . $this->db->quote($year . '-10-01'));
								$year++;
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') < ' . $this->db->quote($year . '-01-01'));
								break;
						}
						break;
					case 'lastquarter':
						// Find out which quarter we are in
						$month = $jdate->__get('month');
						$year = date('Y');
						$quarter = ceil($month / 3);

						if ($quarter == 1)
						{
							$quarter = 4;
							$year--;
						}
						else
						{
							$quarter--;
						}

						switch ($quarter)
						{
							case '1':
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') >= ' . $this->db->quote($year . '-01-01'));
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') < ' . $this->db->quote($year . '-04-01'));
								break;
							case '2':
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') >= ' . $this->db->quote($year . '-04-01'));
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') < ' . $this->db->quote($year . '-07-01'));
								break;
							case '3':
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') >= ' . $this->db->quote($year . '-07-01'));
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') < ' . $this->db->quote($year . '-10-01'));
								break;
							case '4':
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') >= ' . $this->db->quote($year . '-10-01'));
								$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') < ' . $this->db->quote($year++ . '-01-01'));
								break;
						}
						break;
					case 'thisyear':
						$year = date('Y');
						$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') >= ' . $this->db->quote($year . '-01-01'));
						$year++;
						$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') < ' . $this->db->quote($year . '-01-01'));
						break;
					case 'lastyear':
						$year = date('Y');
						$year--;
						$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') >= ' . $this->db->quote($year . '-01-01'));
						$year++;
						$query->where('FROM_UNIXTIME(' . $this->db->quoteName('o.order_created') . ', \'' . $dateformat . '\') < ' . $this->db->quote($year . '-01-01'));
						break;
				}
			}
			else
			{
				// Filter by order date start
				$orderdatestart = $this->template->get('orderdatestart', false);

				if ($orderdatestart)
				{
					$orderdate = strtotime(JFactory::getDate($orderdatestart));
					$query->where(
						$this->db->quoteName('o.order_created') . ' >= ' . $this->db->quote($orderdate)
					);
				}

				// Filter by order date end
				$orderdateend = $this->template->get('orderdateend', false);

				if ($orderdateend)
				{
					$orderdate = strtotime(JFactory::getDate($orderdateend));
					$query->where(
						$this->db->quoteName('o.order_created') . ' <= ' . $this->db->quote($orderdate)
					);
				}

				// Filter by order modified date start
				$ordermdatestart = $this->template->get('ordermdatestart', false);

				if ($ordermdatestart)
				{
					$ordermdate = strtotime(JFactory::getDate($ordermdatestart));
					$query->where(
						$this->db->quoteName('o.order_modified') . ' >= ' . $this->db->quote($ordermdate)
					);
				}

				// Filter by order modified date end
				$ordermdateend = $this->template->get('ordermdateend', false);

				if ($ordermdateend)
				{
					$ordermdate = strtotime(JFactory::getDate($ordermdateend));
					$query->where(
						$this->db->quoteName('o.order_modified') . ' <= ' . $this->db->quote($ordermdate)
					);
				}
			}

			// Filter by order status
			$orderstatus = $this->template->get('orderstatus', false);

			if ($orderstatus && $orderstatus[0] != '')
			{
				$query->where($this->db->quoteName('o.order_status') . ' IN  (\'' . implode("','", $orderstatus) . '\')');
			}

			// Filter by order price start
			$pricestart = $this->template->get('orderpricestart', false, 'float');

			if ($pricestart)
			{
				$query->where($this->db->quoteName('o.order_full_price') . ' >= ' . $pricestart);
			}

			// Filter by order price end
			$priceend = $this->template->get('orderpriceend', false, 'float');

			if ($priceend)
			{
				$query->where($this->db->quoteName('o.order_full_price') . ' <= ' . $priceend);
			}

			// Filter by order user id
			$orderuser = $this->template->get('orderuser', false);

			if ($orderuser && $orderuser[0] != '')
			{
				$query->where($this->db->quoteName('o.order_user_id') . ' IN (\'' . implode("','", $orderuser) . '\')');
			}

			// Filter by order product
			$orderproduct = $this->template->get('orderproduct', false);

			if ($orderproduct && $orderproduct[0] != '')
			{
				$query->where($this->db->quoteName('p.order_product_code') . ' IN (\'' . implode("','", $orderproduct) . '\')');
			}

			// Filter by order currency
			$ordercurrency = $this->template->get('ordercurrency', false);

			if ($ordercurrency && $ordercurrency[0] != '')
			{
				$query->where($this->db->quoteName('o.order_currency_id') . ' IN (\'' . implode("','", $ordercurrency) . '\')');
			}

			// Filter by payment method
			$orderpayment = $this->template->get('orderpayment', false);

			if ($orderpayment && $orderpayment[0] != '')
			{
				$query->where($this->db->quoteName('o.order_payment_id') . ' IN (\'' . implode("','", $orderpayment) . '\')');
			}

			// Group the fields
			$groupby = array_unique($groupby);

			if (!empty($groupby))
			{
				$query->group($groupby);
			}

			// Sort set fields
			$sortby = array_unique($sortby);

			if (!empty($sortby))
			{
				$query->order($sortby);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->csvidb->setQuery($query, $limits['offset'], $limits['limit']);
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->csvidb->getNumRows();

			if ($logcount > 0)
			{
				// Check if we need to split the orderline
				$splitLine = $this->getTemplate()->get('splitorderline', 'jform');

				if ($splitLine)
				{
					// Set the order ID
					$orderId = 0;
				}

				while ($record = $this->csvidb->getRow())
				{
					if ($splitLine)
					{
						// Set the order ID
						if ($orderId == 0)
						{
							$orderId = $record->order_id;
							$emptyLine = false;
						}
					}
					else
					{
						$emptyLine = false;
					}

					$this->log->incrementLinenumber();

					foreach ($exportfields as $field)
					{
						$fieldname = $field->field_name;

						// Set the field value
						if (isset($record->$fieldname))
						{
							$fieldvalue = $record->$fieldname;
						}
						else
						{
							$fieldvalue = '';
						}

						// Process the field
						switch ($fieldname)
						{
							case 'order_discount_price':
							case 'order_full_price':
							case 'order_payment_price':
							case 'order_product_price':
							case 'order_product_shipping_price':
							case 'order_shipping_price':
							case 'user_partner_price':
								if ($fieldvalue)
								{
									$fieldvalue = number_format(
										$fieldvalue,
										$this->template->get('export_price_format_decimal', 2, 'int'),
										$this->template->get('export_price_format_decsep'),
										$this->template->get('export_price_format_thousep')
									);
								}
								break;
							case 'order_modified':
							case 'order_created':
								$dateFormat = $this->template->get('export_date_format', 'd/m/Y H:i:s');

								// Export date according to the format in export template options
								$fieldvalue = date($dateFormat, $fieldvalue);
								break;
							case 'category_path':
								$fieldvalue = $categoryHelper->createCategoryPathById($record->category_id);
								break;
							case 'order_product_options':
								$values = unserialize($record->$fieldname);

								if (is_array($values))
								{
									$exportValues = array();

									foreach ($values as $title => $value)
									{
										if (is_string($title))
										{
											$exportValues[] = $title . ': ' . $value;
										}
									}

									$fieldvalue = implode(', ', $exportValues);
								}

								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldvalue);
					}

					// Output the data
					$this->addExportFields();

					if ($splitLine)
					{
						// Keep track of the order ID
						if ($record->order_id != $orderId)
						{
							$orderId = $record->order_id;
							$emptyLine = true;
						}
					}

					// Output the contents
					$this->writeOutput($emptyLine);

					if ($splitLine)
					{
						$emptyLine = false;
					}
				}
			}
			else
			{
				$this->addExportContent(JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
